/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2023 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.impl.recipe.ingredient.builtin;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.function.Function;

import com.mojang.serialization.Codec;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredient;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredientSerializer;
import net.minecraft.class_1856;
import net.minecraft.class_2540;
import net.minecraft.class_2960;

/**
 * Base class for ALL and ANY ingredients.
 */
abstract class CombinedIngredient implements CustomIngredient {
	protected final List<class_1856> ingredients;

	protected CombinedIngredient(List<class_1856> ingredients) {
		if (ingredients.isEmpty()) {
			throw new IllegalArgumentException("ALL or ANY ingredient must have at least one sub-ingredient");
		}

		this.ingredients = ingredients;
	}

	@Override
	public boolean requiresTesting() {
		for (class_1856 ingredient : ingredients) {
			if (ingredient.requiresTesting()) {
				return true;
			}
		}

		return false;
	}

	List<class_1856> getIngredients() {
		return ingredients;
	}

	static class Serializer<I extends CombinedIngredient> implements CustomIngredientSerializer<I> {
		private final class_2960 identifier;
		private final Function<List<class_1856>, I> factory;
		private final Codec<I> allowEmptyCodec;
		private final Codec<I> disallowEmptyCodec;

		Serializer(class_2960 identifier, Function<List<class_1856>, I> factory, Codec<I> allowEmptyCodec, Codec<I> disallowEmptyCodec) {
			this.identifier = identifier;
			this.factory = factory;
			this.allowEmptyCodec = allowEmptyCodec;
			this.disallowEmptyCodec = disallowEmptyCodec;
		}

		@Override
		public class_2960 getIdentifier() {
			return identifier;
		}

		@Override
		public Codec<I> getCodec(boolean allowEmpty) {
			return allowEmpty ? allowEmptyCodec : disallowEmptyCodec;
		}

		@Override
		public I read(class_2540 buf) {
			int size = buf.method_10816();
			List<class_1856> ingredients = new ArrayList<>(size);

			for (int i = 0; i < size; i++) {
				ingredients.add(class_1856.method_8086(buf));
			}

			return factory.apply(Collections.unmodifiableList(ingredients));
		}

		@Override
		public void write(class_2540 buf, I ingredient) {
			buf.method_10804(ingredient.ingredients.size());

			for (class_1856 value : ingredient.ingredients) {
				value.method_8088(buf);
			}
		}
	}
}
